@@ -8,13 +8,14 @@ Support for results that aren't immediately available.
 Maintainer: Glyph Lefkowitz
 """
 
+import traceback
+import warnings
 from abc import ABC, abstractmethod
 from asyncio import AbstractEventLoop, Future, iscoroutine
 from enum import Enum
 from functools import wraps
-from sys import exc_info, version_info
-import traceback
-from types import GeneratorType, MappingProxyType
+from sys import exc_info
+from types import CoroutineType, GeneratorType, MappingProxyType
 from typing import (
     TYPE_CHECKING,
     Any,
@@ -35,19 +36,23 @@ from typing import (
     cast,
     overload,
 )
-from typing_extensions import Literal
-import warnings
 
 import attr
-
 from incremental import Version
+from typing_extensions import Literal, Protocol
 
 from twisted.internet.interfaces import IDelayedCall, IReactorTime
 from twisted.logger import Logger
-from twisted.python.failure import Failure, _extraneous
 from twisted.python import lockfile
-from twisted.python.compat import cmp, comparable, _PYPY
+from twisted.python.compat import _PYPY, cmp, comparable
 from twisted.python.deprecate import deprecated, warnAboutFunction
+from twisted.python.failure import Failure, _extraneous
+
+
+class _Context(Protocol):
+    def run(self, f: Callable[..., object], *args: object, **kwargs: object) -> object:
+        ...
+
 
 try:
     from contextvars import copy_context as __copy_context
@@ -172,13 +177,22 @@ def maybeDeferred(
     f: Callable[..., _T], *args: object, **kwargs: object
 ) -> "Deferred[_T]":
     """
-    Invoke a function that may or may not return a L{Deferred}.
+    Invoke a function that may or may not return a L{Deferred} or coroutine.
 
-    Call the given function with the given arguments.  If the returned
-    object is a L{Deferred}, return it.  If the returned object is a L{Failure},
-    wrap it with L{fail} and return it.  Otherwise, wrap it in L{succeed} and
-    return it.  If an exception is raised, convert it to a L{Failure}, wrap it
-    in L{fail}, and then return it.
+    Call the given function with the given arguments.  Then:
+
+      - If the returned object is a L{Deferred}, return it.
+
+      - If the returned object is a L{Failure}, wrap it with L{fail} and
+        return it.
+
+      - If the returned object is a L{types.CoroutineType}, wrap it with
+        L{Deferred.fromCoroutine} and return it.
+
+      - Otherwise, wrap it in L{succeed} and return it.
+
+      - If an exception is raised, convert it to a L{Failure}, wrap it in
+        L{fail}, and then return it.
 
     @param f: The callable to invoke
     @param args: The arguments to pass to C{f}
@@ -196,6 +210,25 @@ def maybeDeferred(
         return result
     elif isinstance(result, Failure):
         return fail(result)
+    elif type(result) is CoroutineType:
+        # A note on how we identify this case ...
+        #
+        # inspect.iscoroutinefunction(f) should be the simplest and easiest
+        # way to determine if we want to apply coroutine handling.  However,
+        # the value may be returned by a regular function that calls a
+        # coroutine function and returns its result.  It would be confusing if
+        # cases like this led to different handling of the coroutine (even
+        # though it is a mistake to have a regular function call a coroutine
+        # function to return its result - doing so immediately destroys a
+        # large part of the value of coroutine functions: that they can only
+        # have a coroutine result).
+        #
+        # There are many ways we could inspect ``result`` to determine if it
+        # is a "coroutine" but most of these are mistakes.  The goal is only
+        # to determine whether the value came from ``async def`` or not
+        # because these are the only values we're trying to handle with this
+        # case.  Such values always have exactly one type: CoroutineType.
+        return Deferred.fromCoroutine(result)
     else:
         return succeed(result)
 
@@ -930,7 +963,7 @@ class Deferred(Awaitable[_DeferredResultT]):
         result = getattr(self, "result", _NO_RESULT)
         myID = id(self)
         if self._chainedTo is not None:
-            result = " waiting on Deferred at 0x{:x}".format(id(self._chainedTo))
+            result = f" waiting on Deferred at 0x{id(self._chainedTo):x}"
         elif result is _NO_RESULT:
             result = ""
         else:
@@ -1099,11 +1132,10 @@ class Deferred(Awaitable[_DeferredResultT]):
 
         @raise ValueError: If C{coro} is not a coroutine or generator.
         """
-        # type note: Subclass of "Generator[Deferred[_T], object, _T]" and "GeneratorType" cannot exist
-        if not iscoroutine(coro) and not isinstance(coro, GeneratorType):  # type: ignore[unreachable]
-            raise NotACoroutineError(f"{coro!r} is not a coroutine")
-
-        return _cancellableInlineCallbacks(coro)
+        # asyncio.iscoroutine identifies generators as coroutines, too.
+        if iscoroutine(coro):
+            return _cancellableInlineCallbacks(coro)
+        raise NotACoroutineError(f"{coro!r} is not a coroutine")
 
 
 def ensureDeferred(
@@ -1117,7 +1149,7 @@ def ensureDeferred(
     Schedule the execution of a coroutine that awaits/yields from L{Deferred}s,
     wrapping it in a L{Deferred} that will fire on success/failure of the
     coroutine. If a Deferred is passed to this function, it will be returned
-    directly (mimicing the L{asyncio.ensure_future} function).
+    directly (mimicking the L{asyncio.ensure_future} function).
 
     See L{Deferred.fromCoroutine} for examples of coroutines.
 
@@ -1618,6 +1650,7 @@ def _inlineCallbacks(
         Coroutine[Deferred[_T], object, None],
     ],
     status: _CancellationStatus,
+    context: _Context,
 ) -> None:
     """
     Carry out the work of L{inlineCallbacks}.
@@ -1636,6 +1669,8 @@ def _inlineCallbacks(
         decorated with C{@}L{inlineCallbacks}
 
     @param status: a L{_CancellationStatus} tracking the current status of C{gen}
+
+    @param context: the contextvars context to run `gen` in
     """
     # This function is complicated by the need to prevent unbounded recursion
     # arising from repeatedly yielding immediately ready deferreds.  This while
@@ -1645,8 +1680,8 @@ def _inlineCallbacks(
     # waiting for result?  # result
     waiting: List[Any] = [True, None]
 
-    # Get the current contextvars Context object.
-    current_context = _copy_context()
+    stopIteration: bool = False
+    callbackValue: Any = None
 
     while 1:
         try:
@@ -1654,15 +1689,16 @@ def _inlineCallbacks(
             isFailure = isinstance(result, Failure)
 
             if isFailure:
-                result = current_context.run(
+                result = context.run(
                     cast(Failure, result).throwExceptionIntoGenerator, gen
                 )
             else:
-                result = current_context.run(gen.send, result)
+                result = context.run(gen.send, result)
         except StopIteration as e:
             # fell off the end, or "return" statement
-            status.deferred.callback(getattr(e, "value", None))
-            return
+            stopIteration = True
+            callbackValue = getattr(e, "value", None)
+
         except _DefGen_Return as e:
             # returnValue() was called; time to give a result to the original
             # Deferred.  First though, let's try to identify the potentially
@@ -1682,11 +1718,7 @@ def _inlineCallbacks(
             appCodeTrace = traceback.tb_next
             assert appCodeTrace is not None
 
-            if version_info < (3, 7):
-                # The contextvars backport and our no-op shim add an extra frame.
-                appCodeTrace = appCodeTrace.tb_next
-                assert appCodeTrace is not None
-            elif _PYPY:
+            if _PYPY:
                 # PyPy as of 3.7 adds an extra frame.
                 appCodeTrace = appCodeTrace.tb_next
                 assert appCodeTrace is not None
@@ -1735,12 +1767,20 @@ def _inlineCallbacks(
                     lineno,
                 )
 
-            status.deferred.callback(e.value)
-            return
+            stopIteration = True
+            callbackValue = e.value
+
         except BaseException:
             status.deferred.errback()
             return
 
+        if stopIteration:
+            # Call the callback outside of the exception handler to avoid inappropriate/confusing
+            # "During handling of the above exception, another exception occurred:" if the callback
+            # itself throws an exception.
+            status.deferred.callback(callbackValue)
+            return
+
         if isinstance(result, Deferred):
             # a deferred was yielded, get the result.
             def gotResult(r: object) -> None:
@@ -1748,7 +1788,7 @@ def _inlineCallbacks(
                     waiting[0] = False
                     waiting[1] = r
                 else:
-                    current_context.run(_inlineCallbacks, r, gen, status)
+                    _inlineCallbacks(r, gen, status, context)
 
             result.addBoth(gotResult)
             if waiting[0]:
@@ -1813,7 +1853,7 @@ def _cancellableInlineCallbacks(
 
         return status.deferred
 
-    _inlineCallbacks(None, gen, status)
+    _inlineCallbacks(None, gen, status, _copy_context())
 
     return deferred
 
@@ -1898,15 +1938,12 @@ def inlineCallbacks(
                 "inlineCallbacks requires %r to produce a generator; instead"
                 "caught returnValue being used in a non-generator" % (f,)
             )
-        # type note: FIXME: error suggests type of gen is wrong:
-        #     Subclass of "Generator[Deferred, object, None]" and "GeneratorType"
-        #     cannot exist: would have incompatible method signatures
-        if not isinstance(gen, GeneratorType):  # type: ignore[unreachable]
+        if not isinstance(gen, GeneratorType):
             raise TypeError(
                 "inlineCallbacks requires %r to produce a generator; "
                 "instead got %r" % (f, gen)
             )
-        return _cancellableInlineCallbacks(gen)  # type: ignore[unreachable]
+        return _cancellableInlineCallbacks(gen)
 
     return unwindGenerator
 
