@@ -11,21 +11,18 @@ import errno
 import os
 import re
 import sys
-
 from inspect import getmro
 from io import BytesIO, StringIO
 from typing import Type
-from unittest import expectedFailure
-from unittest import TestCase as StdlibTestCase
+from unittest import TestCase as StdlibTestCase, expectedFailure
 
 from twisted.python import log, reflect
 from twisted.python.failure import Failure
 from twisted.python.reflect import qual
-from twisted.trial import itrial, unittest, runner, reporter, util
-from twisted.trial.reporter import _ExitWrapper, UncleanWarningsReporterWrapper
-from twisted.trial.test import erroneous
-from twisted.trial.unittest import makeTodo, SkipTest, Todo
-from twisted.trial.test import sample
+from twisted.trial import itrial, reporter, runner, unittest, util
+from twisted.trial.reporter import UncleanWarningsReporterWrapper, _ExitWrapper
+from twisted.trial.test import erroneous, sample
+from twisted.trial.unittest import SkipTest, Todo, makeTodo
 
 
 class BrokenStream:
@@ -76,7 +73,7 @@ class StringTest(unittest.SynchronousTestCase):
                     % (line_number, exp.pattern, out),
                 )
             else:
-                raise TypeError("don't know what to do with object {!r}".format(exp))
+                raise TypeError(f"don't know what to do with object {exp!r}")
 
 
 class TestResultTests(unittest.SynchronousTestCase):
@@ -105,6 +102,23 @@ class TestResultTests(unittest.SynchronousTestCase):
         self.assertEqual(excValue, failure.value)
         self.assertEqual(self.failureException, failure.type)
 
+    def test_somethingElse(self):
+        """
+        L{reporter.TestResult.addError} raises L{TypeError} if it is called with
+        an error that is neither a L{sys.exc_info}-like three-tuple nor a
+        L{Failure}.
+        """
+        with self.assertRaises(TypeError):
+            self.result.addError(self, "an error")
+        with self.assertRaises(TypeError):
+            self.result.addError(self, Exception("an error"))
+        with self.assertRaises(TypeError):
+            self.result.addError(
+                self, (Exception, Exception("an error"), None, "extra")
+            )
+        with self.assertRaises(TypeError):
+            self.result.addError(self, (Exception, Exception("an error")))
+
 
 class ReporterRealtimeTests(TestResultTests):
     def setUp(self):
@@ -148,7 +162,7 @@ class ErrorReportingTests(StringTest):
                 "twisted.trial.test.erroneous.FoolishError: "
                 "I am a broken setUp method"
             ),
-            "{}.{}.test_noop".format(cls.__module__, cls.__name__),
+            f"{cls.__module__}.{cls.__name__}.test_noop",
         ]
         self.stringComparison(match, output)
 
@@ -301,7 +315,14 @@ class TracebackHandlingTests(unittest.SynchronousTestCase):
         """
         test = erroneous.TestAsynchronousFail("test_fail")
         frames = self.getErrorFrames(test)
-        self.checkFrames(frames, [("_later", "twisted/trial/test/erroneous")])
+        self.checkFrames(
+            frames,
+            [
+                # This depends on the implementation of test_fail.  Currently
+                # it uses deferLater so we get a frame from the task module.
+                ("cb", "twisted/internet/task"),
+            ],
+        )
 
     def test_noFrames(self):
         result = reporter.Reporter(None)
@@ -947,7 +968,7 @@ class ReporterInterfaceTests(unittest.SynchronousTestCase):
         callable must take the same parameters as L{reporter.Reporter}.
     """
 
-    resultFactory = reporter.Reporter  # type: Type[itrial.IReporter]
+    resultFactory: Type[itrial.IReporter] = reporter.Reporter
 
     def setUp(self):
         self.test = sample.FooTest("test_foo")
@@ -1133,7 +1154,7 @@ class SubunitReporterTests(ReporterInterfaceTests):
     This just tests that the subunit reporter implements the basic interface.
     """
 
-    resultFactory = reporter.SubunitReporter  # type: Type[itrial.IReporter]
+    resultFactory: Type[itrial.IReporter] = reporter.SubunitReporter
 
     def setUp(self):
         if reporter.TestProtocolClient is None:
@@ -1346,7 +1367,7 @@ class SubunitReporterNotInstalledTests(unittest.SynchronousTestCase):
 
 
 class TimingReporterTests(ReporterTests):
-    resultFactory = reporter.TimingTextReporter  # type: Type[itrial.IReporter]
+    resultFactory: Type[itrial.IReporter] = reporter.TimingTextReporter
 
 
 class LoggingReporter(reporter.Reporter):
