@@ -9,23 +9,22 @@ import os
 import pdb
 import sys
 import unittest as pyunit
-
 from io import StringIO
 from typing import List
 
 from zope.interface import implementer
 from zope.interface.verify import verifyObject
 
-from twisted.trial.itrial import IReporter, ITestCase
-from twisted.trial import unittest, runner, reporter, util
-from twisted.trial._asyncrunner import _ForceGarbageCollectionDecorator
+from twisted import plugin
+from twisted.internet import defer
+from twisted.plugins import twisted_trial
 from twisted.python import failure, log, reflect
 from twisted.python.filepath import FilePath
 from twisted.python.reflect import namedAny
 from twisted.scripts import trial
-from twisted.plugins import twisted_trial
-from twisted import plugin
-from twisted.internet import defer
+from twisted.trial import reporter, runner, unittest, util
+from twisted.trial._asyncrunner import _ForceGarbageCollectionDecorator
+from twisted.trial.itrial import IReporter, ITestCase
 
 
 class CapturingDebugger:
@@ -330,38 +329,10 @@ class RunnerTests(unittest.SynchronousTestCase):
         self.original = plugin.getPlugins
         plugin.getPlugins = getPlugins
 
-        self.standardReport = [
-            "startTest",
-            "addSuccess",
-            "stopTest",
-            "startTest",
-            "addSuccess",
-            "stopTest",
-            "startTest",
-            "addSuccess",
-            "stopTest",
-            "startTest",
-            "addSuccess",
-            "stopTest",
-            "startTest",
-            "addSuccess",
-            "stopTest",
-            "startTest",
-            "addSuccess",
-            "stopTest",
-            "startTest",
-            "addSuccess",
-            "stopTest",
-            "startTest",
-            "addSuccess",
-            "stopTest",
-            "startTest",
-            "addSuccess",
-            "stopTest",
-            "startTest",
-            "addSuccess",
-            "stopTest",
-        ]
+        # twisted.trial.test.sample has 10 test cases in it so the "standard"
+        # report expected from running it will have 10 repetitions of this
+        # sequence.
+        self.standardReport = ["startTest", "addSuccess", "stopTest"] * 10
 
     def tearDown(self):
         plugin.getPlugins = self.original
@@ -642,7 +613,7 @@ class UntilFailureTests(unittest.SynchronousTestCase):
         A test case that fails when run 3 times in a row.
         """
 
-        count = []  # type: List[None]
+        count: List[None] = []
 
         def test_foo(self):
             self.count.append(None)
