@@ -15,8 +15,12 @@ from twisted.python.usage import UsageError
 from twisted.scripts import trial
 from twisted.trial import unittest
 from twisted.trial._dist.disttrial import DistTrialRunner
-from twisted.trial.runner import TestLoader
-from twisted.trial.runner import TrialRunner, TestSuite, DestructiveTestSuite
+from twisted.trial.runner import (
+    DestructiveTestSuite,
+    TestLoader,
+    TestSuite,
+    TrialRunner,
+)
 from twisted.trial.test.test_loader import testNames
 
 pyunit = __import__("unittest")
@@ -202,9 +206,7 @@ class TestModuleTests(unittest.SynchronousTestCase):
         try:
             self.config.opt_testmodule(filename)
             self.assertEqual(0, len(self.config["tests"]))
-            self.assertEqual(
-                "File {!r} doesn't exist\n".format(filename), buffy.getvalue()
-            )
+            self.assertEqual(f"File {filename!r} doesn't exist\n", buffy.getvalue())
         finally:
             sys.stderr = stderr
 
@@ -227,9 +229,7 @@ class TestModuleTests(unittest.SynchronousTestCase):
         try:
             self.config.opt_testmodule(moduleName)
             self.assertEqual(0, len(self.config["tests"]))
-            self.assertEqual(
-                "File {!r} doesn't exist\n".format(moduleName), buffy.getvalue()
-            )
+            self.assertEqual(f"File {moduleName!r} doesn't exist\n", buffy.getvalue())
         finally:
             sys.stderr = stderr
 
@@ -307,7 +307,7 @@ class TestModuleTests(unittest.SynchronousTestCase):
         for filename in ["test_script.py", "twisted/trial/test/test_script.py"]:
             self.assertTrue(
                 trial.isTestFile(filename),
-                "{!r} should be a test file".format(filename),
+                f"{filename!r} should be a test file",
             )
         for filename in [
             "twisted/trial/test/moduletest.py",
@@ -316,7 +316,7 @@ class TestModuleTests(unittest.SynchronousTestCase):
         ]:
             self.assertFalse(
                 trial.isTestFile(filename),
-                "{!r} should *not* be a test file".format(filename),
+                f"{filename!r} should *not* be a test file",
             )
 
 
@@ -514,18 +514,6 @@ class OptionsTests(unittest.TestCase):
             "You can't specify --debug-stacktraces when using --jobs", str(error)
         )
 
-    def test_jobsConflictWithExitFirst(self):
-        """
-        C{parseOptions} raises a C{UsageError} when C{--exitfirst} is passed
-        along C{--jobs} as it's not supported yet.
-
-        @see: U{http://twistedmatrix.com/trac/ticket/6436}
-        """
-        error = self.assertRaises(
-            UsageError, self.options.parseOptions, ["--jobs", "4", "--exitfirst"]
-        )
-        self.assertEqual("You can't specify --exitfirst when using --jobs", str(error))
-
     def test_orderConflictWithRandom(self):
         """
         C{parseOptions} raises a C{UsageError} when C{--order} is passed along
@@ -550,13 +538,13 @@ class MakeRunnerTests(unittest.TestCase):
     def test_jobs(self):
         """
         L{_makeRunner} returns a L{DistTrialRunner} instance when the C{--jobs}
-        option is passed, and passes the C{workerNumber} and C{workerArguments}
-        parameters to it.
+        option is passed.  The L{DistTrialRunner} knows how many workers to
+        run and the C{workerArguments} to pass to them.
         """
         self.options.parseOptions(["--jobs", "4", "--force-gc"])
         runner = trial._makeRunner(self.options)
         self.assertIsInstance(runner, DistTrialRunner)
-        self.assertEqual(4, runner._workerNumber)
+        self.assertEqual(4, runner._maxWorkers)
         self.assertEqual(["--force-gc"], runner._workerArguments)
 
     def test_dryRunWithJobs(self):
@@ -806,7 +794,7 @@ class OrderTests(unittest.TestCase):
         pathEntry = package.parent().path
         sys.path.insert(0, pathEntry)
         self.addCleanup(sys.path.remove, pathEntry)
-        from twisted_toptobottom_temp import test_missing
+        from twisted_toptobottom_temp import test_missing  # type: ignore[import]
 
         self.addCleanup(sys.modules.pop, "twisted_toptobottom_temp")
         self.addCleanup(sys.modules.pop, test_missing.__name__)
@@ -859,7 +847,7 @@ class HelpOrderTests(unittest.TestCase):
         msg = "%r with its description not properly described in %r"
         for orderName, (orderDesc, _) in trial._runOrders.items():
             match = re.search(
-                "{}.*{}".format(re.escape(orderName), re.escape(orderDesc)),
+                f"{re.escape(orderName)}.*{re.escape(orderDesc)}",
                 output,
             )
 
